import pycurl
import requests
import sys
import yaml
from concurrent.futures import ThreadPoolExecutor, as_completed, TimeoutError
from io import BytesIO

config = yaml.full_load(open("config.yaml").read())

current_proxy_queue = []
def check_connection(ip, port, proxy_type):
	if proxy_type == "httpsocks":
		current_proxy_type = "http"
	else:
		current_proxy_type = proxy_type
	b = BytesIO()
	c = pycurl.Curl()
	c.setopt(c.URL, config['check_proxies_url'])
	c.setopt(c.PROXY, f"{current_proxy_type}://{ip}:{port}")
	c.setopt(c.WRITEDATA, b)
	c.setopt(c.CONNECTTIMEOUT, config["proxy_timeout"])
	c.setopt(c.TIMEOUT, config["proxy_timeout"] * 2)
	c.setopt(c.SSL_VERIFYPEER, 0)
	c.setopt(c.SSL_VERIFYHOST, 0)
	c.setopt(c.USERAGENT, config["user_agent"])
	try:
		c.perform()
		c.close()
	except pycurl.error as e:
		if e.args[0] == 28:
			return ("inactive", "unknown", proxy_type)
		if e.args[0] == 7 and "Connection refused" in e.args[1]:
			return ("inactive", "unknown", proxy_type)
		if "Proxy CONNECT aborted" in e.args[1]:
			return check_connection(ip, port, "socks5")
		if e.args[0] == 7 and "SOCKS4" in e.args[1]:
			return check_connection(ip, port, "socks5")
		if e.args[0] == 56 and "Connection reset" in e.args[1] and proxy_type != "socks4":
			return check_connection(ip, port, "socks4")
		if e.args[0] == 52 and proxy_type != "socks4":
			return check_connection(ip, port, "socks4")
		if e.args[0] == 35 and proxy_type != "http":
			return check_connection(ip, port, "http")
		else:
			return ("inactive", "unknown", proxy_type)
	except Exception as e:
		print(e)
		return ("inactive", "unknown", proxy_type)
	else:
		proxy_return = b.getvalue().decode(errors="ignore").split(" ", maxsplit=1)
		if len(proxy_return) == 2 and proxy_return[1].rstrip('\n') == config["proxy_string"]:
			print(f"{current_proxy_type}://{ip}:{port} {proxy_return}")
			return ("active", proxy_return[0], current_proxy_type)
		else:
			return ("inactive", "unknown", proxy_type)


def check_proxy(ip, port, exit_ip, proxy_type, comment, p_id):
	base_results = {
		"id": p_id,
		"status": "inactive",
		"entry_ip": ip,
		"exit_ip": exit_ip,
		"proxy_type": proxy_type,
		"port": port,
		"comment": comment,
	}

	current_proxy_queue.append(f"{ip}:{port}")
	proxy_status, proxy_exit, found_proxy_type = check_connection(ip, port, proxy_type)
	current_proxy_queue.remove(f"{ip}:{port}")
	if len(current_proxy_queue) < 10:
		print(current_proxy_queue)
	if proxy_status == "active":
		base_results["status"] = "active"
		base_results["exit_ip"] = proxy_exit
		base_results["proxy_type"] = found_proxy_type

	return base_results

# Make sure we've got an internet connection first
test = requests.get(config['check_proxies_url'])
test_split = test.text.split(" ", maxsplit=1)
if len(test_split) != 2 or test_split[1].rstrip('\n') != config["proxy_string"]:
	sys.exit(1)

proxy_results = []
amount = config["proxy_amount"]
r = requests.get(f"{config['get_proxies_url']}?amount={amount}&proxy_type=http&proxy_type=socks5&proxy_type=socks4&proxy_type=https&proxy_type=httpsocks&timedelta={config['timedelta']}&max_num_failures={config['max_num_failures']}")
current_proxies = r.json()
all_threads = {}
with ThreadPoolExecutor(max_workers=config["proxy_threads"]) as executor:
	for proxy in current_proxies["proxies"]:
		all_threads[executor.submit(check_proxy, proxy["ip"], proxy["port"], proxy["exit_ip"], proxy["proxy_type"], proxy["comment"], proxy["id"])] = proxy["id"]
	for future in as_completed(all_threads):
		try:
			proxy_results.append(future.result(timeout=30))
		except TimeoutError as e:
			print(e)

data = {"proxies": proxy_results}
submit = requests.post(f"{config['update_proxies_url']}", json=data)
